import pytest

import salt.modules.pf as pf
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {pf: {}}


def test_enable_when_disabled():
    """
    Tests enabling pf when it's not enabled yet.
    """
    ret = {}
    ret["stderr"] = "pf enabled"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.enable()["changes"]


def test_enable_when_enabled():
    """
    Tests enabling pf when it already enabled.
    """
    ret = {}
    ret["stderr"] = "pfctl: pf already enabled"
    ret["retcode"] = 1
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert not pf.enable()["changes"]


def test_disable_when_enabled():
    """
    Tests disabling pf when it's enabled.
    """
    ret = {}
    ret["stderr"] = "pf disabled"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.disable()["changes"]


def test_disable_when_disabled():
    """
    Tests disabling pf when it already disabled.
    """
    ret = {}
    ret["stderr"] = "pfctl: pf not enabled"
    ret["retcode"] = 1
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert not pf.disable()["changes"]


def test_loglevel_freebsd():
    """
    Tests setting a loglevel.
    """
    ret = {}
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
        pf.__grains__, {"os": "FreeBSD"}
    ):
        res = pf.loglevel("urgent")
        mock_cmd.assert_called_once_with(
            "pfctl -x urgent", output_loglevel="trace", python_shell=False
        )
        assert res["changes"]


def test_loglevel_openbsd():
    """
    Tests setting a loglevel.
    """
    ret = {}
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}), patch.dict(
        pf.__grains__, {"os": "OpenBSD"}
    ):
        res = pf.loglevel("crit")
        mock_cmd.assert_called_once_with(
            "pfctl -x crit", output_loglevel="trace", python_shell=False
        )
        assert res["changes"]


def test_load():
    """
    Tests loading ruleset.
    """
    ret = {}
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        res = pf.load()
        mock_cmd.assert_called_once_with(
            ["pfctl", "-f", "/etc/pf.conf"],
            output_loglevel="trace",
            python_shell=False,
        )
        assert res["changes"]


def test_load_noop():
    """
    Tests evaluating but not actually loading ruleset.
    """
    ret = {}
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        res = pf.load(noop=True)
        mock_cmd.assert_called_once_with(
            ["pfctl", "-f", "/etc/pf.conf", "-n"],
            output_loglevel="trace",
            python_shell=False,
        )
        assert not res["changes"]


def test_flush():
    """
    Tests a regular flush command.
    """
    ret = {}
    ret["stderr"] = "pf: statistics cleared"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        res = pf.flush("info")
        mock_cmd.assert_called_once_with(
            "pfctl -v -F info", output_loglevel="trace", python_shell=False
        )
        assert res["changes"]


def test_flush_capital():
    """
    Tests a flush command starting with a capital letter.
    """
    ret = {}
    ret["stderr"] = "2 tables cleared"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        res = pf.flush("tables")
        mock_cmd.assert_called_once_with(
            "pfctl -v -F Tables", output_loglevel="trace", python_shell=False
        )
        assert res["changes"]


def test_flush_without_changes():
    """
    Tests a flush command that has no changes.
    """
    ret = {}
    ret["stderr"] = "0 tables cleared"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert not pf.flush("tables")["changes"]


def test_table():
    """
    Tests a regular table command.
    """
    ret = {}
    ret["stderr"] = "42 addresses deleted"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("flush", table="bad_hosts")["changes"]


def test_table_expire():
    """
    Tests the table expire command.
    """
    ret = {}
    ret["stderr"] = "1/1 addresses expired."
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("expire", table="bad_hosts", number=300)["changes"]


def test_table_add_addresses():
    """
    Tests adding addresses to a table.
    """
    ret = {}
    ret["stderr"] = "2/2 addressess added."
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("add", table="bad_hosts", addresses=["1.2.3.4", "5.6.7.8"])[
            "changes"
        ]


def test_table_delete_addresses():
    """
    Tests deleting addresses in a table.
    """
    ret = {}
    ret["stderr"] = "2/2 addressess deleted."
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("delete", table="bad_hosts", addresses=["1.2.3.4", "5.6.7.8"])[
            "changes"
        ]


def test_table_test_address():
    """
    Tests testing addresses in a table.
    """
    ret = {}
    ret["stderr"] = "1/2 addressess match."
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("test", table="bad_hosts", addresses=["1.2.3.4"])["matches"]


def test_table_no_changes():
    """
    Tests a table command that has no changes.
    """
    ret = {}
    ret["stderr"] = "0/1 addresses expired."
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert not pf.table("expire", table="bad_hosts", number=300)["changes"]


def test_table_show():
    """
    Tests showing table contents.
    """
    ret = {}
    ret["stdout"] = "1.2.3.4\n5.6.7.8"
    ret["retcode"] = 0
    expected = ["1.2.3.4", "5.6.7.8"]
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("show", table="bad_hosts")["comment"] == expected


def test_table_zero():
    """
    Tests clearing all the statistics of a table.
    """
    ret = {}
    ret["stderr"] = "42 addresses has been cleared"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.table("zero", table="bad_hosts")["changes"]


def test_show_rules():
    """
    Tests show rules command.
    """
    ret = {}
    ret["stdout"] = "block return\npass"
    ret["retcode"] = 0
    expected = ["block return", "pass"]
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.show("rules")["comment"] == expected


def test_show_states():
    """
    Tests show states command.
    """
    ret = {}
    ret["stdout"] = "all udp 192.168.1.1:3478\n"
    ret["retcode"] = 0
    expected = ["all udp 192.168.1.1:3478", ""]
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        assert pf.show("states")["comment"] == expected


def test_show_tables():
    """
    Tests show tables command.
    """
    ret = {}
    ret["stdout"] = "bad_hosts"
    ret["retcode"] = 0
    mock_cmd = MagicMock(return_value=ret)
    with patch.dict(pf.__salt__, {"cmd.run_all": mock_cmd}):
        res = pf.show("tables")
        mock_cmd.assert_called_once_with(
            "pfctl -s Tables", output_loglevel="trace", python_shell=False
        )
        assert not res["changes"]



import logging

import pytest

import salt.modules.solarispkg as solarispkg
from tests.support.mock import ANY, MagicMock, call, patch

log = logging.getLogger(__name__)


class ListPackages:
    def __init__(self, iteration=0):
        self._iteration = iteration

    def __call__(self):
        # Package information returned two lines per package. On even-offset
        # lines, the package name is in the first column. On odd-offset lines, the
        # package version is in the second column.
        pkg_lists = [
            {
                "SUNWbzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWgzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWzlib": "11.10.0,REV=2005.01.08.01.09",
            },
            {
                "SUNWbzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWgzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWzip": "11.10.0,REV=2005.01.08.01.09",
                "SUNWzlib": "11.10.0,REV=2005.01.08.01.09",
                "SUNWbashS": "11.10.0,REV=2005.01.08.01.09",
            },
        ]
        pkgs = pkg_lists[self._iteration]
        self._iteration = (self._iteration + 1) % 2
        return pkgs


@pytest.fixture
def configure_loader_modules():

    return {
        solarispkg: {
            "__grains__": {
                "osarch": "sparcv9",
                "os_family": "Solaris",
                "osmajorrelease": 10,
                "kernelrelease": 5.10,
            },
        },
    }


def test_list_pkgs():
    """
    Test for listing installed packages.
    """

    def _add_data(data, key, value):
        data[key] = value

    pkg_info_out = [
        "SUNWbzip                          The bzip compression utility",
        "                                  (i386) 11.10.0,REV=2005.01.08.01.09",
        "SUNWgzip                          The GNU Zip (gzip) compression utility",
        "                                  (i386) 11.10.0,REV=2005.01.08.01.09",
        "SUNWzip                           The Info-Zip (zip) compression utility",
        "                                  (i386) 11.10.0,REV=2005.01.08.01.09",
        "SUNWzlib                          The Zip compression library",
        "                                  (i386) 11.10.0,REV=2005.01.08.01.09",
    ]
    run_mock = MagicMock(return_value="\n".join(pkg_info_out))
    patches = {
        "cmd.run": run_mock,
        "pkg_resource.add_pkg": _add_data,
        "pkg_resource.sort_pkglist": MagicMock(),
        "pkg_resource.stringify": MagicMock(),
    }
    with patch.dict(solarispkg.__salt__, patches):
        pkgs = solarispkg.list_pkgs()
        assert pkgs == {
            "SUNWbzip": "11.10.0,REV=2005.01.08.01.09",
            "SUNWgzip": "11.10.0,REV=2005.01.08.01.09",
            "SUNWzip": "11.10.0,REV=2005.01.08.01.09",
            "SUNWzlib": "11.10.0,REV=2005.01.08.01.09",
        }
    run_mock.assert_called_once_with(
        "/usr/bin/pkginfo -x", output_loglevel="trace", python_shell=False
    )


def test_install_single_named_package():
    """
    Test installing a single package
    - a single package SUNWbashS from current drive
    """

    install_target = "SUNWbashS"
    parsed_targets = (
        {install_target: None},
        "repository",
    )
    cmd_out = {
        "retcode": 0,
        "stdout": "",
        "stderr": "",
    }
    run_all_mock = MagicMock(return_value=cmd_out)
    patches = {
        "cmd.run_all": run_all_mock,
        "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
        "pkg_resource.stringify": MagicMock(),
        "pkg_resource.sort_pkglist": MagicMock(),
        "cmd.retcode": MagicMock(return_value=False),
    }

    with patch.dict(solarispkg.__salt__, patches):
        with patch("salt.modules.solarispkg.list_pkgs", ListPackages()):
            added = solarispkg.install(
                install_target,
                sources=[{install_target: "tests/pytest/unit/module/sol10_pkg/bashs"}],
                refresh=False,
            )
            expected = {
                "SUNWbashS": {"new": "11.10.0,REV=2005.01.08.01.09", "old": ""},
            }
            assert added == expected

    expected_calls = [
        call(
            ["/usr/sbin/pkgadd", "-n", "-a", ANY, "-d", install_target, "all"],
            output_loglevel="trace",
            python_shell=False,
        ),
    ]
    run_all_mock.assert_has_calls(expected_calls, any_order=True)
    assert run_all_mock.call_count == 1


def test_install_single_named_package_global_zone_boolean():
    """
    Test installing a single package
    - a single package SUNWbashS from current drive
    """

    install_target = "SUNWbashS"
    parsed_targets = (
        {install_target: None},
        "repository",
    )
    cmd_out = {
        "retcode": 0,
        "stdout": "",
        "stderr": "",
    }
    run_all_mock = MagicMock(return_value=cmd_out)
    patches = {
        "cmd.run_all": run_all_mock,
        "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
        "pkg_resource.stringify": MagicMock(),
        "pkg_resource.sort_pkglist": MagicMock(),
        "cmd.retcode": MagicMock(return_value=False),
    }

    with patch.dict(solarispkg.__salt__, patches):
        with patch("salt.modules.solarispkg.list_pkgs", ListPackages()):
            added = solarispkg.install(
                install_target,
                sources=[{install_target: "tests/pytest/unit/module/sol10_pkg/bashs"}],
                refresh=False,
                current_zone_only=True,
            )
            expected = {
                "SUNWbashS": {"new": "11.10.0,REV=2005.01.08.01.09", "old": ""},
            }
            assert added == expected

    expected_calls = [
        call(
            [
                "/usr/sbin/pkgadd",
                "-n",
                "-a",
                ANY,
                "-G ",
                "-d",
                install_target,
                "all",
            ],
            output_loglevel="trace",
            python_shell=False,
        ),
    ]
    run_all_mock.assert_has_calls(expected_calls, any_order=True)
    assert run_all_mock.call_count == 1


def test_install_single_named_package_global_zone_text():
    """
    Test installing a single package
    - a single package SUNWbashS from current drive
    """

    install_target = "SUNWbashS"
    parsed_targets = (
        {install_target: None},
        "repository",
    )
    cmd_out = {
        "retcode": 0,
        "stdout": "",
        "stderr": "",
    }
    run_all_mock = MagicMock(return_value=cmd_out)
    patches = {
        "cmd.run_all": run_all_mock,
        "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
        "pkg_resource.stringify": MagicMock(),
        "pkg_resource.sort_pkglist": MagicMock(),
        "cmd.retcode": MagicMock(return_value=False),
    }

    with patch.dict(solarispkg.__salt__, patches):
        with patch("salt.modules.solarispkg.list_pkgs", ListPackages()):
            added = solarispkg.install(
                install_target,
                sources=[{install_target: "tests/pytest/unit/module/sol10_pkg/bashs"}],
                refresh=False,
                current_zone_only="True",
            )
            expected = {
                "SUNWbashS": {"new": "11.10.0,REV=2005.01.08.01.09", "old": ""},
            }
            assert added == expected

    expected_calls = [
        call(
            [
                "/usr/sbin/pkgadd",
                "-n",
                "-a",
                ANY,
                "-G ",
                "-d",
                install_target,
                "all",
            ],
            output_loglevel="trace",
            python_shell=False,
        ),
    ]
    run_all_mock.assert_has_calls(expected_calls, any_order=True)
    assert run_all_mock.call_count == 1


def test_remove_single_named_package():
    """
    Test removing a single specific named package
    - a single package SUNWbashS
    """

    install_target = "SUNWbashS"
    parsed_targets = (
        {install_target: None},
        "repository",
    )
    cmd_out = {
        "retcode": 0,
        "stdout": "",
        "stderr": "",
    }
    run_all_mock = MagicMock(return_value=cmd_out)
    patches = {
        "cmd.run_all": run_all_mock,
        "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
        "pkg_resource.stringify": MagicMock(),
        "pkg_resource.sort_pkglist": MagicMock(),
        "cmd.retcode": MagicMock(return_value=False),
    }

    with patch.dict(solarispkg.__salt__, patches):
        with patch("salt.modules.solarispkg.list_pkgs", ListPackages(1)):
            added = solarispkg.remove(install_target, refresh=False)
            expected = {
                "SUNWbashS": {"new": "", "old": "11.10.0,REV=2005.01.08.01.09"},
            }
            assert added == expected

    expected_calls = [
        call(
            ["/usr/sbin/pkgrm", "-n", "-a", ANY, install_target],
            output_loglevel="trace",
            python_shell=False,
        ),
    ]
    run_all_mock.assert_has_calls(expected_calls, any_order=True)
    assert run_all_mock.call_count == 1



import pytest

import salt.modules.cmdmod as cmdmod
import salt.utils.path

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
]


@pytest.fixture(params=["powershell", "pwsh"])
def shell(request):
    """
    This will run the test on powershell and powershell core (pwsh). If
    powershell core is not installed that test run will be skipped
    """

    if request.param == "pwsh" and salt.utils.path.which("pwsh") is None:
        pytest.skip("Powershell 7 Not Present")
    return request.param


@pytest.fixture(scope="module")
def account():
    with pytest.helpers.create_account() as _account:
        yield _account


@pytest.fixture
def issue_56195(state_tree, account):
    tmpdir = f"C:\\Users\\{account.username}\\AppData\\Local\\Temp"
    contents = """[CmdLetBinding()]
Param(
    [SecureString] $SecureString
)
$Credential = New-Object System.Net.NetworkCredential("DummyId", $SecureString)
$Credential.Password
"""
    with pytest.helpers.temp_file("test.ps1", contents, tmpdir) as f:
        yield str(f), account


def test_args(issue_56195):
    """
    Ensure that powershell processes an inline script with args where the args
    contain powershell that needs to be rendered
    """
    (script, _) = issue_56195
    password = "i like cheese"
    args = (
        "-SecureString (ConvertTo-SecureString -String '{}' -AsPlainText -Force)"
        " -ErrorAction Stop".format(password)
    )
    # https://github.com/PowerShell/PowerShell/issues/18530
    cmd = f'$env:PSModulePath=""; {script} {args}'
    ret = cmdmod.powershell(cmd, args=args, saltenv="base")
    assert ret == password


def test_args_runas(issue_56195):
    """
    Ensure that powershell with runas processes an inline script with args where
    the args contain powershell that needs to be rendered
    """
    (script, account) = issue_56195
    password = "i like cheese"
    args = (
        "-SecureString (ConvertTo-SecureString -String '{}' -AsPlainText -Force)"
        " -ErrorAction Stop".format(password)
    )
    # https://github.com/PowerShell/PowerShell/issues/18530
    cmd = f'$env:PSModulePath=""; {script} {args}'
    ret = cmdmod.powershell(
        cmd,
        args=args,
        runas=account.username,
        password=account.password,
        saltenv="base",
    )
    assert ret == password


@pytest.mark.parametrize(
    "cmd, expected, encode_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        ('Write-Output "Encoded Foo"', "Encoded Foo", True),
        (["Write-Output", '"Encoded Foo"'], "Encoded Foo", True),
        ("$a=\"Plain\";$b=' Foo';Write-Output ${a}${b}", "Plain Foo", False),
        ("(Write-Output Foo)", "Foo", False),
        ("& Write-Output Foo", "Foo", False),
    ],
)
def test_powershell(shell, cmd, expected, encode_cmd):
    """
    Test cmd.powershell
    """
    ret = cmdmod.powershell(cmd=cmd, encode_cmd=encode_cmd, shell=shell)
    assert ret == expected


@pytest.mark.parametrize(
    "cmd, expected, encode_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        ('Write-Output "Encoded Foo"', "Encoded Foo", True),
        (["Write-Output", '"Encoded Foo"'], "Encoded Foo", True),
        ("$a=\"Plain\";$b=' Foo';Write-Output ${a}${b}", "Plain Foo", False),
        ("(Write-Output Foo)", "Foo", False),
        ("& Write-Output Foo", "Foo", False),
    ],
)
def test_powershell_runas(shell, account, cmd, expected, encode_cmd):
    """
    Test cmd.powershell with runas
    """
    ret = cmdmod.powershell(
        cmd=cmd,
        encode_cmd=encode_cmd,
        shell=shell,
        runas=account.username,
        password=account.password,
    )
    assert ret == expected


@pytest.mark.parametrize(
    "cmd, expected, encode_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        ('Write-Output "Encoded Foo"', "Encoded Foo", True),
        (["Write-Output", '"Encoded Foo"'], "Encoded Foo", True),
    ],
)
def test_powershell_all(shell, cmd, expected, encode_cmd):
    """
    Test cmd.powershell_all. `encode_cmd` takes the passed command and encodes
    it. Different from encoded_command where it's receiving an already encoded
    command
    """
    ret = cmdmod.powershell_all(cmd=cmd, encode_cmd=encode_cmd, shell=shell)
    assert isinstance(ret["pid"], int)
    assert ret["retcode"] == 0
    assert ret["stderr"] == ""
    assert ret["result"] == expected


@pytest.mark.parametrize(
    "cmd, expected, encode_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        ('Write-Output "Encoded Foo"', "Encoded Foo", True),
        (["Write-Output", '"Encoded Foo"'], "Encoded Foo", True),
    ],
)
def test_powershell_all_runas(shell, account, cmd, expected, encode_cmd):
    """
    Test cmd.powershell_all with runas. `encode_cmd` takes the passed command
    and encodes it. Different from encoded_command where it's receiving an
    already encoded command
    """
    ret = cmdmod.powershell_all(
        cmd=cmd,
        encode_cmd=encode_cmd,
        shell=shell,
        runas=account.username,
        password=account.password,
    )
    assert isinstance(ret["pid"], int)
    assert ret["retcode"] == 0
    assert ret["stderr"] == ""
    assert ret["result"] == expected


@pytest.mark.parametrize(
    "cmd, expected, encoded_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        (
            "VwByAGkAdABlAC0ASABvAHMAdAAgACcARQBuAGMAbwBkAGUAZAAgAEYAbwBvACcA",
            "Encoded Foo",
            True,
        ),
    ],
)
def test_cmd_run_all_powershell(shell, cmd, expected, encoded_cmd):
    """
    Ensure that cmd.run_all supports running shell='powershell'
    """
    ret = cmdmod.run_all(cmd=cmd, shell=shell, encoded_cmd=encoded_cmd)
    assert ret["stdout"] == expected


@pytest.mark.parametrize(
    "cmd, expected, encoded_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        (
            "VwByAGkAdABlAC0ASABvAHMAdAAgACcARQBuAGMAbwBkAGUAZAAgAEYAbwBvACcA",
            "Encoded Foo",
            True,
        ),
    ],
)
def test_cmd_run_all_powershell_runas(shell, account, cmd, expected, encoded_cmd):
    """
    Ensure that cmd.run_all with runas supports running shell='powershell'
    """
    ret = cmdmod.run_all(
        cmd=cmd,
        shell=shell,
        encoded_cmd=encoded_cmd,
        runas=account.username,
        password=account.password,
    )
    assert ret["stdout"] == expected


@pytest.mark.parametrize(
    "cmd, expected, encoded_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        (
            "VwByAGkAdABlAC0ASABvAHMAdAAgACcARQBuAGMAbwBkAGUAZAAgAEYAbwBvACcA",
            "Encoded Foo",
            True,
        ),
    ],
)
def test_cmd_run_encoded_cmd(shell, cmd, expected, encoded_cmd):
    """
    Ensure that cmd.run supports running shell='powershell'
    """
    ret = cmdmod.run(
        cmd=cmd, shell=shell, encoded_cmd=encoded_cmd, redirect_stderr=False
    )
    assert ret == expected


@pytest.mark.parametrize(
    "cmd, expected, encoded_cmd",
    [
        ("Write-Output Foo", "Foo", False),
        (["Write-Output", "Foo"], "Foo", False),
        (
            "VwByAGkAdABlAC0ASABvAHMAdAAgACcARQBuAGMAbwBkAGUAZAAgAEYAbwBvACcA",
            "Encoded Foo",
            True,
        ),
    ],
)
def test_cmd_run_encoded_cmd_runas(shell, account, cmd, expected, encoded_cmd):
    """
    Ensure that cmd.run with runas supports running shell='powershell'
    """
    ret = cmdmod.run(
        cmd=cmd,
        shell=shell,
        encoded_cmd=encoded_cmd,
        redirect_stderr=False,
        runas=account.username,
        password=account.password,
    )
    assert ret == expected



import logging

import pytest

log = logging.getLogger(__name__)

pytestmark = [
    pytest.mark.core_test,
    pytest.mark.skip_if_not_root,
    pytest.mark.destructive_test,
    pytest.mark.skip_on_windows,
]


@pytest.fixture(scope="module")
def salt_auth_account_1(salt_auth_account_1_factory):
    with salt_auth_account_1_factory as account:
        yield account


@pytest.fixture(scope="module")
def salt_auth_account_2(salt_auth_account_2_factory):
    with salt_auth_account_2_factory as account:
        yield account


def test_pam_auth_valid_user(salt_minion, salt_cli, salt_auth_account_1):
    """
    test that pam auth mechanism works with a valid user
    """
    # test user auth against pam
    ret = salt_cli.run(
        "-a",
        "pam",
        "--username",
        salt_auth_account_1.username,
        "--password",
        salt_auth_account_1.password,
        "test.ping",
        minion_tgt=salt_minion.id,
    )
    assert ret.returncode == 0
    assert ret.data is True


def test_pam_auth_invalid_user(salt_minion, salt_cli):
    """
    test pam auth mechanism errors for an invalid user
    """
    ret = salt_cli.run(
        "-a",
        "pam",
        "--username",
        "nouser",
        "--password",
        "1234",
        "test.ping",
        minion_tgt=salt_minion.id,
    )
    assert ret.stdout == "Authentication error occurred."


def test_pam_auth_valid_group(salt_minion, salt_cli, salt_auth_account_2):
    """
    test that pam auth mechanism works for a valid group
    """
    # test group auth against pam: saltadm is not configured in
    # external_auth, but saltops is and saldadm is a member of saltops
    ret = salt_cli.run(
        "-a",
        "pam",
        "--username",
        salt_auth_account_2.username,
        "--password",
        salt_auth_account_2.password,
        "test.ping",
        minion_tgt=salt_minion.id,
    )
    assert ret.returncode == 0
    assert ret.data is True



"""
:codeauthor: Shane Lee <slee@saltstack.com>
"""

import copy

import pytest

import salt.config
import salt.loader
import salt.states.win_lgpo as win_lgpo
import salt.utils.platform
import salt.utils.stringutils
from tests.support.mock import patch


@pytest.fixture
def configure_loader_modules(minion_opts):
    utils = salt.loader.utils(minion_opts)
    modules = salt.loader.minion_mods(minion_opts, utils=utils)
    return {
        win_lgpo: {
            "__opts__": copy.deepcopy(minion_opts),
            "__salt__": modules,
            "__utils__": utils,
        }
    }


@pytest.fixture
def policy_clear():
    # Make sure policy is not set to begin with, unsets it after test
    try:
        computer_policy = {"Point and Print Restrictions": "Not Configured"}
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        yield
    finally:
        computer_policy = {"Point and Print Restrictions": "Not Configured"}
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            win_lgpo.set_(name="test_state", computer_policy=computer_policy)


@pytest.fixture
def policy_set():
    # Make sure policy is set to begin with, unsets it after test
    try:
        computer_policy = {
            "Point and Print Restrictions": {
                "Users can only point and print to these servers": True,
                "Enter fully qualified server names separated by semicolons": (
                    "fakeserver1;fakeserver2"
                ),
                "Users can only point and print to machines in their forest": True,
                "When installing drivers for a new connection": (
                    "Show warning and elevation prompt"
                ),
                "When updating drivers for an existing connection": "Show warning only",
            }
        }
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        yield
    finally:
        computer_policy = {"Point and Print Restrictions": "Not Configured"}
        with patch.dict(win_lgpo.__opts__, {"test": False}):
            win_lgpo.set_(name="test_state", computer_policy=computer_policy)


def test__compare_policies_string():
    """
    ``_compare_policies`` should only return ``True`` when the string values
    are the same. All other scenarios should return ``False``
    """
    compare_string = "Salty test"
    # Same
    assert win_lgpo._compare_policies(compare_string, compare_string)
    # Different
    assert not win_lgpo._compare_policies(compare_string, "Not the same")
    # List
    assert not win_lgpo._compare_policies(compare_string, ["item1", "item2"])
    # Dict
    assert not win_lgpo._compare_policies(compare_string, {"key": "value"})
    # None
    assert not win_lgpo._compare_policies(compare_string, None)


def test__compare_policies_list():
    """
    ``_compare_policies`` should only return ``True`` when the lists are the
    same. All other scenarios should return ``False``
    """
    compare_list = ["Salty", "test"]
    # Same
    assert win_lgpo._compare_policies(compare_list, compare_list)
    # Different
    assert not win_lgpo._compare_policies(compare_list, ["Not", "the", "same"])
    # String
    assert not win_lgpo._compare_policies(compare_list, "Not a list")
    # Dict
    assert not win_lgpo._compare_policies(compare_list, {"key": "value"})
    # None
    assert not win_lgpo._compare_policies(compare_list, None)


def test__compare_policies_dict():
    """
    ``_compare_policies`` should only return ``True`` when the dicts are the
    same. All other scenarios should return ``False``
    """
    compare_dict = {"Salty": "test"}
    # Same
    assert win_lgpo._compare_policies(compare_dict, compare_dict)
    # Different
    assert not win_lgpo._compare_policies(compare_dict, {"key": "value"})
    # String
    assert not win_lgpo._compare_policies(compare_dict, "Not a dict")
    # List
    assert not win_lgpo._compare_policies(compare_dict, ["Not", "a", "dict"])
    # None
    assert not win_lgpo._compare_policies(compare_dict, None)


def test__compare_policies_integer():
    """
    ``_compare_policies`` should only return ``True`` when the integer
    values are the same. All other scenarios should return ``False``
    """
    compare_integer = 1
    # Same
    assert win_lgpo._compare_policies(compare_integer, compare_integer)
    # Different
    assert not win_lgpo._compare_policies(compare_integer, 0)
    # List
    assert not win_lgpo._compare_policies(compare_integer, ["item1", "item2"])
    # Dict
    assert not win_lgpo._compare_policies(compare_integer, {"key": "value"})
    # None
    assert not win_lgpo._compare_policies(compare_integer, None)


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_current_element_naming_style(policy_clear):
    """
    Ensure that current naming style works properly.
    """
    computer_policy = {
        "Point and Print Restrictions": {
            "Users can only point and print to these servers": True,
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "Users can only point and print to machines in their forest": True,
            "When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "When updating drivers for an existing connection": "Show warning only",
        }
    }
    with patch.dict(win_lgpo.__opts__, {"test": False}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
        result = win_lgpo._convert_to_unicode(result)
    expected = {
        "Point and Print Restrictions": {
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "Users can only point and print to machines in their forest": True,
            "Users can only point and print to these servers": True,
            "When updating drivers for an existing connection": "Show warning only",
        }
    }
    assert result["changes"]["new"]["Computer Configuration"] == expected


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_old_element_naming_style(policy_clear):
    """
    Ensure that the old naming style is converted to new and a warning is
    returned
    """
    computer_policy = {
        "Point and Print Restrictions": {
            "Users can only point and print to these servers": True,
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "Users can only point and print to machines in their forest": True,
            # Here's the old one
            "Security Prompts: When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "When updating drivers for an existing connection": "Show warning only",
        }
    }

    with patch.dict(win_lgpo.__opts__, {"test": False}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
    assert result["changes"] == {}
    expected = (
        "The LGPO module changed the way it gets policy element names.\n"
        '"Security Prompts: When installing drivers for a new connection" is no longer valid.\n'
        'Please use "When installing drivers for a new connection" instead.'
    )
    assert result["comment"] == expected


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_invalid_elements():
    computer_policy = {
        "Point and Print Restrictions": {
            "Invalid element spongebob": True,
            "Invalid element squidward": False,
        }
    }

    with patch.dict(win_lgpo.__opts__, {"test": False}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
    expected = {
        "changes": {},
        "comment": (
            "Invalid element name: Invalid element squidward\n"
            "Invalid element name: Invalid element spongebob"
        ),
        "name": "test_state",
        "result": False,
    }
    assert result["changes"] == expected["changes"]
    assert "Invalid element squidward" in result["comment"]
    assert "Invalid element spongebob" in result["comment"]
    assert not expected["result"]


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_current_element_naming_style_true(policy_set):
    """
    Test current naming style with test=True
    """
    computer_policy = {
        "Point and Print Restrictions": {
            "Users can only point and print to these servers": True,
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "Users can only point and print to machines in their forest": True,
            "When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "When updating drivers for an existing connection": "Show warning only",
        }
    }
    with patch.dict(win_lgpo.__opts__, {"test": True}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
    expected = {
        "changes": {},
        "comment": "All specified policies are properly configured",
    }
    assert result["changes"] == expected["changes"]
    assert result["result"]
    assert result["comment"] == expected["comment"]


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_old_element_naming_style_true(policy_set):
    """
    Test old naming style with test=True. Should not make changes but return a
    warning
    """
    computer_policy = {
        "Point and Print Restrictions": {
            "Users can only point and print to these servers": True,
            "Enter fully qualified server names separated by semicolons": (
                "fakeserver1;fakeserver2"
            ),
            "Users can only point and print to machines in their forest": True,
            # Here's the old one
            "Security Prompts: When installing drivers for a new connection": (
                "Show warning and elevation prompt"
            ),
            "When updating drivers for an existing connection": "Show warning only",
        }
    }
    with patch.dict(win_lgpo.__opts__, {"test": True}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
    expected = {
        "changes": {},
        "comment": (
            "The LGPO module changed the way it gets policy element names.\n"
            '"Security Prompts: When installing drivers for a new connection" is no longer valid.\n'
            'Please use "When installing drivers for a new connection" instead.'
        ),
    }
    assert result["changes"] == expected["changes"]
    assert not result["result"]
    assert result["comment"] == expected["comment"]


@pytest.mark.skip_unless_on_windows
@pytest.mark.destructive_test
@pytest.mark.slow_test
def test_invalid_elements_true():
    computer_policy = {
        "Point and Print Restrictions": {
            "Invalid element spongebob": True,
            "Invalid element squidward": False,
        }
    }

    with patch.dict(win_lgpo.__opts__, {"test": True}):
        result = win_lgpo.set_(name="test_state", computer_policy=computer_policy)
    expected = {
        "changes": {},
        "comment": (
            "Invalid element name: Invalid element squidward\n"
            "Invalid element name: Invalid element spongebob"
        ),
        "name": "test_state",
        "result": False,
    }
    assert result["changes"] == expected["changes"]
    assert "Invalid element squidward" in result["comment"]
    assert "Invalid element spongebob" in result["comment"]
    assert not expected["result"]



"""
Utilies for beacons
"""

import copy


def remove_hidden_options(config, whitelist):
    """
    Remove any hidden options not whitelisted
    """
    for entry in copy.copy(config):
        for func in entry:
            if func.startswith("_") and func not in whitelist:
                config.remove(entry)
    return config


def list_to_dict(config):
    """
    Convert list based beacon configuration
    into a dictionary.
    """
    _config = {}
    list(map(_config.update, config))
    return _config



import os

import pytest

import salt.modules.kmod as kmod
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, mock_open, patch


@pytest.fixture
def configure_loader_modules():
    return {kmod: {}}


def test_available():
    """
    Tests return a list of all available kernel modules
    """
    with patch("salt.modules.kmod.available", MagicMock(return_value=["kvm"])):
        assert kmod.available() == ["kvm"]


def test_check_available():
    """
    Tests if the specified kernel module is available
    """
    with patch("salt.modules.kmod.available", MagicMock(return_value=["kvm"])):
        assert kmod.check_available("kvm") is True


def test_lsmod():
    """
    Tests return information about currently loaded modules
    """
    ret_str = """Module                  Size  Used by
    kvm_intel             233472  0
    """
    expected = [{"size": "233472", "module": "kvm_intel", "depcount": "0", "deps": []}]
    mock_cmd = MagicMock(return_value=ret_str)
    with patch(
        "salt.utils.path.which", MagicMock(side_effect=[None, "/sbin/lsmod"])
    ), patch.dict(kmod.__salt__, {"cmd.run": mock_cmd}):
        with pytest.raises(CommandExecutionError):
            kmod.lsmod()
        assert kmod.lsmod() == expected


@pytest.mark.skipif(
    not os.path.isfile("/etc/modules"), reason="/etc/modules not present"
)
def test_mod_list():
    """
    Tests return a list of the loaded module names
    """
    with patch(
        "salt.modules.kmod._get_modules_conf",
        MagicMock(return_value="/etc/modules"),
    ), patch("salt.utils.files.fopen", mock_open(read_data="lp")):
        assert kmod.mod_list(True) == ["lp"]

    mock_ret = [{"size": 100, "module": None, "depcount": 10, "deps": None}]
    with patch("salt.modules.kmod.lsmod", MagicMock(return_value=mock_ret)):
        assert kmod.mod_list(False) == [None]


def test_load():
    """
    Tests to loads specified kernel module.
    """
    mod = "cheese"
    err_msg = "Module too moldy, refusing to load"
    mock_persist = MagicMock(return_value={mod})
    mock_lsmod = MagicMock(
        return_value=[{"size": 100, "module": None, "depcount": 10, "deps": None}]
    )
    mock_run_all_0 = MagicMock(return_value={"retcode": 0})
    mock_run_all_1 = MagicMock(return_value={"retcode": 1, "stderr": err_msg})

    with patch("salt.modules.kmod._set_persistent_module", mock_persist):
        with patch(
            "salt.utils.path.which",
            MagicMock(side_effect=[None, "/sbin/modprobe", "/sbin/modprobe"]),
        ), patch("salt.modules.kmod.lsmod", mock_lsmod):
            with patch.dict(
                kmod.__salt__, {"cmd.run_all": mock_run_all_0}
            ), pytest.raises(CommandExecutionError):
                kmod.load(mod, True)

            with patch.dict(kmod.__salt__, {"cmd.run_all": mock_run_all_0}):
                assert kmod.load(mod, True) == [mod]

            with patch.dict(kmod.__salt__, {"cmd.run_all": mock_run_all_1}):
                assert kmod.load(mod) == f"Error loading module {mod}: {err_msg}"


def test_is_loaded():
    """
    Tests if specified kernel module is loaded.
    """
    with patch("salt.modules.kmod.mod_list", MagicMock(return_value={"lp"})):
        assert kmod.is_loaded("lp") is True


def test_remove():
    """
    Tests to remove the specified kernel module
    """
    mod = "cheese"
    err_msg = "Cannot find module: it has been eaten"
    mock_persist = MagicMock(return_value={mod})
    mock_lsmod = MagicMock(
        return_value=[{"size": 100, "module": None, "depcount": 10, "deps": None}]
    )
    mock_run_all_0 = MagicMock(return_value={"retcode": 0})
    mock_run_all_1 = MagicMock(return_value={"retcode": 1, "stderr": err_msg})

    with patch("salt.modules.kmod._remove_persistent_module", mock_persist):
        with patch(
            "salt.utils.path.which",
            MagicMock(side_effect=[None, "/sbin/rmmod", "/sbin/rmmod", "/sbin/rmmod"]),
        ), patch("salt.modules.kmod.lsmod", mock_lsmod):
            with patch.dict(kmod.__salt__, {"cmd.run_all": mock_run_all_0}):
                with pytest.raises(CommandExecutionError):
                    kmod.remove(mod)

                assert kmod.remove(mod, True) == [mod]

                assert kmod.remove(mod) == []

            with patch.dict(kmod.__salt__, {"cmd.run_all": mock_run_all_1}):
                assert (
                    kmod.remove(mod, True) == f"Error removing module {mod}: {err_msg}"
                )



import os

import pytest

import salt.utils.path

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.destructive_test,
    pytest.mark.slow_test,
]


@pytest.fixture(params=["powershell", "pwsh"])
def shell(request):
    """
    This will run the test on powershell and powershell core (pwsh). If
    powershell core is not installed that test run will be skipped
    """

    if request.param == "pwsh" and salt.utils.path.which("pwsh") is None:
        pytest.skip("Powershell 7 Not Present")
    return request.param


def test_cmd_run_unless_true(shell, cmd):
    # We need a directory that we know exists that has stuff in it
    win_dir = os.getenv("WINDIR")
    ret = cmd.run(name="echo foo", unless=f"ls {win_dir}", shell=shell)
    assert ret.filtered["result"] is True
    assert ret.filtered["name"] == "echo foo"
    assert ret.filtered["comment"] == "unless condition is true"
    assert ret.filtered["changes"] == {}


def test_cmd_run_unless_false(shell, cmd):
    # We need a directory that we know does not exist
    win_dir = "C:\\This\\Dir\\Does\\Not\\Exist"
    ret = cmd.run(name="echo foo", unless=f"ls {win_dir}", shell=shell)
    assert ret.filtered["result"] is True
    assert ret.filtered["name"] == "echo foo"
    assert ret.filtered["comment"] == 'Command "echo foo" run'
    assert ret.filtered["changes"]["stdout"] == "foo"


def test_cmd_run_onlyif_true(shell, cmd):
    # We need a directory that we know exists that has stuff in it
    win_dir = os.getenv("WINDIR")
    ret = cmd.run(name="echo foo", onlyif=f"ls {win_dir}", shell=shell)
    assert ret.filtered["result"] is True
    assert ret.filtered["name"] == "echo foo"
    assert ret.filtered["comment"] == 'Command "echo foo" run'
    assert ret.filtered["changes"]["stdout"] == "foo"


def test_cmd_run_onlyif_false(shell, cmd):
    # We need a directory that we know does not exist
    win_dir = "C:\\This\\Dir\\Does\\Not\\Exist"
    ret = cmd.run(name="echo foo", onlyif=f"ls {win_dir}", shell=shell)
    assert ret.filtered["result"] is True
    assert ret.filtered["name"] == "echo foo"
    assert ret.filtered["comment"] == "onlyif condition is false"
    assert ret.filtered["changes"] == {}



"""
Tests for salt.utils.jinja
"""

import os

import pytest
from jinja2 import Environment, TemplateNotFound, exceptions

# dateutils is needed so that the strftime jinja filter is loaded
import salt.utils.dateutils  # pylint: disable=unused-import
import salt.utils.files  # pylint: disable=unused-import
import salt.utils.json  # pylint: disable=unused-import
import salt.utils.stringutils  # pylint: disable=unused-import
import salt.utils.yaml  # pylint: disable=unused-import
from salt.utils.jinja import SaltCacheLoader
from tests.support.mock import MagicMock, call, patch


@pytest.fixture
def minion_opts(tmp_path, minion_opts):
    minion_opts.update(
        {
            "file_buffer_size": 1048576,
            "cachedir": str(tmp_path),
            "file_roots": {"test": [str(tmp_path / "files" / "test")]},
            "pillar_roots": {"test": [str(tmp_path / "pillar" / "test")]},
            "extension_modules": os.path.join(
                os.path.dirname(os.path.abspath(__file__)), "extmods"
            ),
        }
    )
    return minion_opts


@pytest.fixture
def hello_simple(template_dir):
    contents = """world
"""

    with pytest.helpers.temp_file(
        "hello_simple", directory=template_dir, contents=contents
    ) as hello_simple_filename:
        yield hello_simple_filename


@pytest.fixture
def hello_include(template_dir):
    contents = """{% include 'hello_import' -%}"""

    with pytest.helpers.temp_file(
        "hello_include", directory=template_dir, contents=contents
    ) as hello_include_filename:
        yield hello_include_filename


@pytest.fixture
def relative_dir(template_dir):
    relative_dir = template_dir / "relative"
    relative_dir.mkdir()
    return relative_dir


@pytest.fixture
def relative_rhello(relative_dir):
    contents = """{% from './rmacro' import rmacro with context -%}
{{ rmacro('Hey') ~ rmacro(a|default('a'), b|default('b')) }}
"""

    with pytest.helpers.temp_file(
        "rhello", directory=relative_dir, contents=contents
    ) as relative_rhello:
        yield relative_rhello


@pytest.fixture
def relative_rmacro(relative_dir):
    contents = """{% from '../macro' import mymacro with context %}
{% macro rmacro(greeting, greetee='world') -%}
{{ mymacro(greeting, greetee) }}
{%- endmacro %}
"""

    with pytest.helpers.temp_file(
        "rmacro", directory=relative_dir, contents=contents
    ) as relative_rmacro:
        yield relative_rmacro


@pytest.fixture
def relative_rescape(relative_dir):
    contents = """{% import '../../rescape' as xfail -%}
"""

    with pytest.helpers.temp_file(
        "rescape", directory=relative_dir, contents=contents
    ) as relative_rescape:
        yield relative_rescape


@pytest.fixture
def get_loader(mock_file_client, minion_opts):
    def run_command(opts=None, saltenv="base", **kwargs):
        """
        Now that we instantiate the client in the __init__, we need to mock it
        """
        if opts is None:
            opts = minion_opts
        mock_file_client.opts = opts
        loader = SaltCacheLoader(opts, saltenv, _file_client=mock_file_client, **kwargs)
        # Create a mock file client and attach it to the loader
        return loader

    return run_command


def get_test_saltenv(get_loader):
    """
    Setup a simple jinja test environment
    """
    loader = get_loader(saltenv="test")
    jinja = Environment(loader=loader)
    return loader._file_client, jinja


def test_searchpath(minion_opts, get_loader, tmp_path):
    """
    The searchpath is based on the cachedir option and the saltenv parameter
    """
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    assert loader.searchpath == minion_opts["file_roots"][saltenv]


def test_searchpath_pillar_rend(minion_opts, get_loader):
    """
    The searchpath is based on the pillar_rend if it is True
    """
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv, pillar_rend=True)
    assert loader.searchpath == minion_opts["pillar_roots"][saltenv]


def test_searchpath_bad_pillar_rend(minion_opts, get_loader):
    """
    The searchpath is based on the pillar_rend if it is True
    """
    saltenv = "bad_env"
    loader = get_loader(opts=minion_opts, saltenv=saltenv, pillar_rend=True)
    assert loader.searchpath == []


def test_mockclient(minion_opts, template_dir, hello_simple, get_loader):
    """
    A MockFileClient is used that records all file requests normally sent
    to the master.
    """
    loader = get_loader(opts=minion_opts, saltenv="test")
    res = loader.get_source(None, "hello_simple")
    assert len(res) == 3
    # res[0] on Windows is unicode and use os.linesep so it works cross OS
    assert str(res[0]) == "world" + os.linesep
    assert res[1] == str(hello_simple)
    assert res[2](), "Template up to date?"
    assert loader._file_client.requests
    assert loader._file_client.requests[0]["path"] == "salt://hello_simple"


def test_import(get_loader, hello_import):
    """
    You can import and use macros from other files
    """
    fc, jinja = get_test_saltenv(get_loader)
    result = jinja.get_template("hello_import").render()
    assert result == "Hey world !a b !"
    assert len(fc.requests) == 2
    assert fc.requests[0]["path"] == "salt://hello_import"
    assert fc.requests[1]["path"] == "salt://macro"


def test_relative_import(
    get_loader, relative_rhello, relative_rmacro, relative_rescape, macro_template
):
    """
    You can import using relative paths
    issue-13889
    """
    fc, jinja = get_test_saltenv(get_loader)
    tmpl = jinja.get_template(os.path.join("relative", "rhello"))
    result = tmpl.render()
    assert result == "Hey world !a b !"
    assert len(fc.requests) == 3
    assert fc.requests[0]["path"] == "salt://relative/rhello"
    assert fc.requests[1]["path"] == "salt://relative/rmacro"
    assert fc.requests[2]["path"] == "salt://macro"
    # This must fail when rendered: attempts to import from outside file root
    template = jinja.get_template("relative/rescape")
    pytest.raises(exceptions.TemplateNotFound, template.render)


def test_include(get_loader, hello_include, hello_import):
    """
    You can also include a template that imports and uses macros
    """
    fc, jinja = get_test_saltenv(get_loader)
    result = jinja.get_template("hello_include").render()
    assert result == "Hey world !a b !"
    assert len(fc.requests) == 3
    assert fc.requests[0]["path"] == "salt://hello_include"
    assert fc.requests[1]["path"] == "salt://hello_import"
    assert fc.requests[2]["path"] == "salt://macro"


def test_include_context(get_loader, hello_include, hello_import):
    """
    Context variables are passes to the included template by default.
    """
    _, jinja = get_test_saltenv(get_loader)
    result = jinja.get_template("hello_include").render(a="Hi", b="Salt")
    assert result == "Hey world !Hi Salt !"


def test_cached_file_client(get_loader, minion_opts):
    """
    Multiple instantiations of SaltCacheLoader use the cached file client
    """
    with patch("salt.channel.client.ReqChannel.factory", MagicMock()):
        loader_a = SaltCacheLoader(minion_opts)
        loader_b = SaltCacheLoader(minion_opts)
    assert loader_a._file_client is loader_b._file_client


def test_file_client_kwarg(minion_opts, mock_file_client):
    """
    A file client can be passed to SaltCacheLoader overriding the any
    cached file client
    """
    mock_file_client.opts = minion_opts
    loader = SaltCacheLoader(minion_opts, _file_client=mock_file_client)
    assert loader._file_client is mock_file_client


def test_cache_loader_passed_file_client(minion_opts, mock_file_client):
    """
    The shudown method can be called without raising an exception when the
    file_client does not have a destroy method
    """
    # Test SaltCacheLoader creating and destroying the file client created
    file_client = MagicMock()
    with patch("salt.fileclient.get_file_client", return_value=file_client):
        loader = SaltCacheLoader(minion_opts)
        assert loader._file_client is None
        with loader:
            assert loader._file_client is file_client
        assert loader._file_client is None
        assert file_client.mock_calls == [call.destroy()]

    # Test SaltCacheLoader reusing the file client passed
    file_client = MagicMock()
    file_client.opts = {"file_roots": minion_opts["file_roots"]}
    with patch("salt.fileclient.get_file_client", return_value=MagicMock()):
        loader = SaltCacheLoader(minion_opts, _file_client=file_client)
        assert loader._file_client is file_client
        with loader:
            assert loader._file_client is file_client
        assert loader._file_client is file_client
        assert file_client.mock_calls == []

    # Test SaltCacheLoader creating a client even though a file client was
    # passed because the "file_roots" option is different, and, as such,
    # the destroy method on the new file client is called, but not on the
    # file client passed in.
    file_client = MagicMock()
    file_client.opts = {"file_roots": ""}
    new_file_client = MagicMock()
    with patch("salt.fileclient.get_file_client", return_value=new_file_client):
        loader = SaltCacheLoader(minion_opts, _file_client=file_client)
        assert loader._file_client is file_client
        with loader:
            assert loader._file_client is not file_client
            assert loader._file_client is new_file_client
        assert loader._file_client is None
        assert file_client.mock_calls == []
        assert new_file_client.mock_calls == [call.destroy()]


def test_check_cache_miss(get_loader, minion_opts, hello_simple):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    with patch.object(loader, "cached", []):
        with patch.object(loader, "cache_file") as cache_mock:
            loader.check_cache(str(hello_simple))
            cache_mock.assert_called_once()


def test_check_cache_hit(get_loader, minion_opts, hello_simple):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    with patch.object(loader, "cached", [str(hello_simple)]):
        with patch.object(loader, "cache_file") as cache_mock:
            loader.check_cache(str(hello_simple))
            cache_mock.assert_not_called()


def test_get_source_no_environment(
    get_loader, minion_opts, relative_rhello, relative_dir
):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    with pytest.raises(TemplateNotFound):
        loader.get_source(None, str(".." / relative_rhello.relative_to(relative_dir)))


def test_get_source_relative_no_tpldir(
    get_loader, minion_opts, relative_rhello, relative_dir
):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    with pytest.raises(TemplateNotFound):
        loader.get_source(
            MagicMock(globals={}), str(".." / relative_rhello.relative_to(relative_dir))
        )


def test_get_source_template_doesnt_exist(get_loader, minion_opts):
    saltenv = "test"
    fake_path = "fake_path"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    with pytest.raises(TemplateNotFound):
        loader.get_source(None, fake_path)


def test_get_source_template_removed(get_loader, minion_opts, hello_simple):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    contents, filepath, uptodate = loader.get_source(None, str(hello_simple))
    hello_simple.unlink()
    assert uptodate() is False


def test_no_destroy_method_on_file_client(get_loader, minion_opts):
    saltenv = "test"
    loader = get_loader(opts=minion_opts, saltenv=saltenv)
    loader._close_file_client = True
    # This should fail silently, thus no error catching
    loader.destroy()



"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>

    Test cases for salt.modules.nagios
"""

import os

import pytest

import salt.modules.nagios as nagios
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {nagios: {}}


def test_run():
    """
    Test for Run nagios plugin and return all
     the data execution with cmd.run
    """
    with patch.object(nagios, "_execute_cmd", return_value="A"):
        assert nagios.run("plugin") == "A"


def test_retcode():
    """
    Test for Run one nagios plugin and return retcode of the execution
    """
    with patch.object(nagios, "_execute_cmd", return_value="A"):
        assert nagios.retcode("plugin", key_name="key") == {"key": {"status": "A"}}


def test_run_all():
    """
    Test for Run nagios plugin and return all
     the data execution with cmd.run_all
    """
    with patch.object(nagios, "_execute_cmd", return_value="A"):
        assert nagios.run_all("plugin") == "A"


def test_retcode_pillar():
    """
    Test for Run one or more nagios plugins from pillar data and
     get the result of cmd.retcode
    """
    with patch.dict(nagios.__salt__, {"pillar.get": MagicMock(return_value={})}):
        assert nagios.retcode_pillar("pillar_name") == {}


def test_run_pillar():
    """
    Test for Run one or more nagios plugins from pillar data
     and get the result of cmd.run
    """
    with patch.object(nagios, "_execute_pillar", return_value="A"):
        assert nagios.run_pillar("pillar") == "A"


def test_run_all_pillar():
    """
    Test for Run one or more nagios plugins from pillar data
     and get the result of cmd.run
    """
    with patch.object(nagios, "_execute_pillar", return_value="A"):
        assert nagios.run_all_pillar("pillar") == "A"


def test_list_plugins():
    """
    Test for List all the nagios plugins
    """
    with patch.object(os, "listdir", return_value=[]):
        assert nagios.list_plugins() == []



import os
import pathlib
import re
import textwrap
import types

import pytest

import salt.utils.hashutils
import salt.utils.platform
import salt.utils.versions
from salt.utils.versions import Version

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_if_binaries_missing("patch"),
]


class Patches:
    def __init__(self, webserver):
        self.webserver = webserver
        self.webserver_root = pathlib.Path(webserver.root)
        self.numbers_patch_name = "numbers.patch"
        self.numbers_patch_template_name = self.numbers_patch_name + ".jinja"
        self.numbers_patch_path = "patches/" + self.numbers_patch_name
        self.numbers_patch_template_path = "patches/" + self.numbers_patch_template_name
        self.numbers_patch = "salt://" + self.numbers_patch_path
        self.numbers_patch_template = "salt://" + self.numbers_patch_template_path
        self.numbers_patch_http = self.webserver.url(self.numbers_patch_path)
        self.numbers_patch_template_http = self.webserver.url(
            self.numbers_patch_template_path
        )

        self.math_patch_name = "math.patch"
        self.math_patch_template_name = self.math_patch_name + ".jinja"
        self.math_patch_path = "patches/" + self.math_patch_name
        self.math_patch_template_path = "patches/" + self.math_patch_template_name
        self.math_patch = "salt://" + self.math_patch_path
        self.math_patch_template = "salt://" + self.math_patch_template_path
        self.math_patch_http = self.webserver.url(self.math_patch_path)
        self.math_patch_template_http = self.webserver.url(
            self.math_patch_template_path
        )

        self.all_patch_name = "all.patch"
        self.all_patch_template_name = self.all_patch_name + ".jinja"
        self.all_patch_path = "patches/" + self.all_patch_name
        self.all_patch_template_path = "patches/" + self.all_patch_template_name
        self.all_patch = "salt://" + self.all_patch_path
        self.all_patch_template = "salt://" + self.all_patch_template_path
        self.all_patch_http = self.webserver.url(self.all_patch_path)
        self.all_patch_template_http = self.webserver.url(self.all_patch_template_path)

        patches_dir = self.webserver_root / "patches"
        patches_dir.mkdir()
        numbers_patch = patches_dir / self.numbers_patch_name
        numbers_patch.write_text(
            textwrap.dedent(
                """\
                --- a/foo/numbers.txt	2018-04-09 18:43:58.014272504 -0500
                +++ b/foo/numbers.txt	2018-04-09 18:44:46.487905044 -0500
                @@ -1,7 +1,7 @@
                -one
                -two
                 three
                +two
                +one

                -1
                -2
                 3
                +2
                +1
                """
            )
        )
        numbers_patch_template = patches_dir / self.numbers_patch_template_name
        numbers_patch_template.write_text(
            numbers_patch.read_text().replace("+two", "+{{ two }}")
        )
        math_patch = patches_dir / self.math_patch_name
        math_patch.write_text(
            textwrap.dedent(
                """\
                --- a/foo/bar/math.txt	2018-04-09 18:43:52.883205365 -0500
                +++ b/foo/bar/math.txt	2018-04-09 18:44:58.525061654 -0500
                @@ -1,3 +1,3 @@
                -Five plus five is ten
                +5 + 5 = 10

                -Four squared is sixteen
                +4² = 16
                """
            )
        )
        math_patch_template = patches_dir / self.math_patch_template_name
        math_patch_template.write_text(
            math_patch.read_text().replace("= 10", "= {{ ten }}")
        )
        all_patch = patches_dir / self.all_patch_name
        all_patch.write_text(math_patch.read_text() + numbers_patch.read_text())
        all_patch_template = patches_dir / self.all_patch_template_name
        all_patch_template.write_text(
            math_patch_template.read_text() + numbers_patch_template.read_text()
        )
        self.numbers_patch_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.numbers_patch_name)
        )
        self.numbers_patch_template_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.numbers_patch_template_name)
        )
        self.math_patch_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.math_patch_name)
        )
        self.math_patch_template_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.math_patch_template_name)
        )
        self.all_patch_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.all_patch_name)
        )
        self.all_patch_template_hash = salt.utils.hashutils.get_hash(
            str(patches_dir / self.all_patch_template_name)
        )


@pytest.fixture(scope="module")
def _check_min_patch_version(shell):
    min_patch_ver = Version("2.6")
    ret = shell.run("patch", "--version")
    assert ret.returncode == 0
    version = ret.stdout.strip().splitlines()[0].split()[-1]
    if Version(version) < min_patch_ver:
        pytest.xfail(
            "Minimum version of patch not found, expecting {}, found {}".format(
                min_patch_ver, version
            )
        )


@pytest.fixture(scope="module")
def patches(webserver, _check_min_patch_version):
    yield Patches(webserver)


@pytest.fixture
def files(tmp_path):
    numbers_contents = """
    one
    two
    three

    1
    2
    3
    """
    math_contents = """
    Five plus five is ten

    Four squared is sixteen
    """
    numbers_file = pytest.helpers.temp_file(
        "numbers.txt", directory=tmp_path / "foo", contents=numbers_contents
    )
    math_file = pytest.helpers.temp_file(
        "math.txt", directory=tmp_path / "foo" / "bar", contents=math_contents
    )
    with numbers_file as numbers_file, math_file as math_file:
        yield types.SimpleNamespace(
            numbers=numbers_file, math=math_file, base_dir=tmp_path
        )


def test_patch_single_file(file, files, patches):
    """
    Test file.patch using a patch applied to a single file
    """
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch)
    assert ret.result is True
    assert ret.changes
    assert ret.comment == "Patch successfully applied"

    # Re-run the state, should succeed and there should be a message about
    # a partially-applied hunk.
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch)
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Patch was already applied"


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
@pytest.mark.skip_on_darwin(
    reason="Failing and previously skipped because patch wasn't >= 2.6"
)
def test_patch_directory(file, files, patches):
    """
    Test file.patch using a patch applied to a directory, with changes
    spanning multiple files.
    """
    ret = file.patch(name=str(files.base_dir), source=patches.all_patch, strip=1)
    assert ret.result is True
    assert ret.changes
    assert ret.comment == "Patch successfully applied"

    # Re-run the state, should succeed and there should be a message about
    # a partially-applied hunk.
    ret = file.patch(name=str(files.base_dir), source=patches.all_patch, strip=1)
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Patch was already applied"


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
@pytest.mark.skip_on_darwin(
    reason="Failing and previously skipped because patch wasn't >= 2.6"
)
def test_patch_strip_parsing(file, files, patches, subtests):
    """
    Test that we successfuly parse -p/--strip when included in the options
    """
    # Run the state using -p1
    with subtests.test("options='-p1'"):
        ret = file.patch(
            name=str(files.base_dir), source=patches.all_patch, options="-p1"
        )
        assert ret.result is True
        assert ret.changes
        assert ret.comment == "Patch successfully applied"

    # Re-run the state using --strip=1
    with subtests.test("options='--strip=1'"):
        ret = file.patch(
            name=str(files.base_dir), source=patches.all_patch, options="--strip=1"
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"

    # Re-run the state using --strip 1
    with subtests.test("options='--strip=1'"):
        ret = file.patch(
            name=str(files.base_dir), source=patches.all_patch, options="--strip=1"
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"


def test_patch_saltenv(file, files, patches):
    """
    Test that we attempt to download the patch from a non-base saltenv
    """
    # This state will fail because we don't have a patch file in that
    # environment, but that is OK, we just want to test that we're looking
    # in an environment other than base.
    ret = file.patch(name=str(files.math), source=patches.math_patch, saltenv="prod")
    assert ret.result is False
    assert ret.comment == "Source file {} not found in saltenv 'prod'".format(
        patches.math_patch
    )


def test_patch_single_file_failure(file, tmp_path, files, patches):
    """
    Test file.patch using a patch applied to a single file. This tests a
    failed patch.
    """
    # Empty the file to ensure that the patch doesn't apply cleanly
    files.numbers.write_text("")

    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch)
    assert ret.result is False
    assert "Patch would not apply cleanly" in ret.comment

    # Test the reject_file option and ensure that the rejects are written
    # to the path specified.
    reject_file = tmp_path / "rejected"
    ret = file.patch(
        name=str(files.numbers),
        source=patches.numbers_patch,
        reject_file=str(reject_file),
        strip=1,
    )
    assert ret.result is False
    assert "Patch would not apply cleanly" in ret.comment
    if salt.utils.platform.is_windows():
        assert_fpath = f".*{reject_file.name}"
    else:
        assert_fpath = reject_file
    assert re.search(f"saving rejects to (file )?{assert_fpath}", ret.comment)


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
def test_patch_directory_failure(file, tmp_path, files, patches):
    """
    Test file.patch using a patch applied to a directory, with changes
    spanning multiple files.
    """
    # Empty the file to ensure that the patch doesn't apply cleanly
    files.numbers.write_text("")

    ret = file.patch(name=str(files.base_dir), source=patches.all_patch, strip=1)
    assert ret.result is False
    assert "Patch would not apply cleanly" in ret.comment

    # Test the reject_file option and ensure that the rejects are written
    # to the path specified.
    reject_file = tmp_path / "rejected"
    ret = file.patch(
        name=str(files.base_dir),
        source=patches.all_patch,
        reject_file=str(reject_file),
        strip=1,
    )
    assert ret.result is False
    assert "Patch would not apply cleanly" in ret.comment
    if salt.utils.platform.is_windows():
        assert_fpath = f".*{reject_file.name}"
    else:
        assert_fpath = reject_file
    assert re.search(f"saving rejects to (file )?{assert_fpath}", ret.comment)


def test_patch_single_file_remote_source(file, files, patches, subtests):
    """
    Test file.patch using a patch applied to a single file, with the patch
    coming from a remote source.
    """
    # Try without a source_hash and without skip_verify=True, this should
    # fail with a message about the source_hash
    with subtests.test("source_hash=None"):
        ret = file.patch(name=str(files.math), source=patches.math_patch_http)
        assert ret.result is False
        assert "Unable to verify upstream hash" in ret.comment

    # Re-run the state with a source hash, it should now succeed
    with subtests.test("source_hash!=None"):
        ret = file.patch(
            name=str(files.math),
            source=patches.math_patch_http,
            source_hash=patches.math_patch_hash,
        )
        assert ret.result is True
        assert ret.changes
        assert ret.comment == "Patch successfully applied"

    # Re-run again, this time with no hash and skip_verify=True to test
    # skipping hash verification
    with subtests.test("source_hash=None and skip_verify=True"):
        ret = file.patch(
            name=str(files.math), source=patches.math_patch_http, skip_verify=True
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
@pytest.mark.skip_on_darwin(
    reason="Failing and previously skipped because patch wasn't >= 2.6"
)
def test_patch_directory_remote_source(file, files, patches, subtests):
    """
    Test file.patch using a patch applied to a directory, with changes
    spanning multiple files, and the patch file coming from a remote
    source.
    """
    # Try without a source_hash and without skip_verify=True, this should
    # fail with a message about the source_hash
    with subtests.test("source_hash=None"):
        ret = file.patch(
            name=str(files.base_dir), source=patches.all_patch_http, strip=1
        )
        assert ret.result is False
        assert "Unable to verify upstream hash" in ret.comment

    # Re-run the state with a source hash, it should now succeed
    with subtests.test("source_hash!=None"):
        ret = file.patch(
            name=str(files.base_dir),
            source=patches.all_patch_http,
            source_hash=patches.all_patch_hash,
            strip=1,
        )
        assert ret.result is True
        assert ret.changes
        assert ret.comment == "Patch successfully applied"

    # Re-run again, this time with no hash and skip_verify=True to test
    # skipping hash verification
    with subtests.test("source_hash=None and skip_verify=True"):
        ret = file.patch(
            name=str(files.base_dir),
            source=patches.all_patch_http,
            skip_verify=True,
            strip=1,
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"


def test_patch_single_file_template(file, files, patches):
    """
    Test file.patch using a patch applied to a single file, with jinja
    templating applied to the patch file.
    """
    ret = file.patch(
        name=str(files.numbers),
        source=patches.numbers_patch_template,
        template="jinja",
        context={"two": "two", "ten": 10},
    )
    assert ret.result is True
    assert ret.comment == "Patch successfully applied"

    # Re-run the state, should succeed and there should be a message about
    # a partially-applied hunk.
    ret = file.patch(
        name=str(files.numbers),
        source=patches.numbers_patch_template,
        template="jinja",
        context={"two": "two", "ten": 10},
    )
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Patch was already applied"


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
@pytest.mark.skip_on_darwin(
    reason="Failing and previously skipped because patch wasn't >= 2.6"
)
def test_patch_directory_template(file, files, patches):
    """
    Test file.patch using a patch applied to a directory, with changes
    spanning multiple files, and with jinja templating applied to the patch
    file.
    """
    if salt.utils.platform.is_windows() and os.environ.get("PYTHONUTF8", "0") == "0":
        pytest.skip("Test will fail if PYTHONUTF8=1 is not set on windows")
    ret = file.patch(
        name=str(files.base_dir),
        source=patches.all_patch_template,
        template="jinja",
        context={"two": "two", "ten": 10},
        strip=1,
    )
    assert ret.result is True
    assert ret.changes
    assert ret.comment == "Patch successfully applied"

    # Re-run the state, should succeed and there should be a message about
    # a partially-applied hunk.
    ret = file.patch(
        name=str(files.base_dir),
        source=patches.all_patch_template,
        template="jinja",
        context={"two": "two", "ten": 10},
        strip=1,
    )
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Patch was already applied"


def test_patch_single_file_remote_source_template(file, files, patches, subtests):
    """
    Test file.patch using a patch applied to a single file, with the patch
    coming from a remote source.
    """
    if salt.utils.platform.is_windows() and os.environ.get("PYTHONUTF8", "0") == "0":
        pytest.skip("Test will fail if PYTHONUTF8=1 is not set on windows")
    # Try without a source_hash and without skip_verify=True, this should
    # fail with a message about the source_hash
    with subtests.test("source_hash=None and skip_verify=False"):
        ret = file.patch(
            name=str(files.math),
            source=patches.math_patch_template_http,
            template="jinja",
            context={"two": "two", "ten": 10},
        )
        assert ret.result is False
        assert not ret.changes
        assert "Unable to verify upstream hash" in ret.comment

    # Re-run the state with a source hash, it should now succeed
    with subtests.test("source_hash!=None"):
        ret = file.patch(
            name=str(files.math),
            source=patches.math_patch_template_http,
            source_hash=patches.math_patch_template_hash,
            template="jinja",
            context={"two": "two", "ten": 10},
        )
        assert ret.result is True
        assert ret.changes
        assert ret.comment == "Patch successfully applied"
    with subtests.test("source_hash=None and skip_verify=True"):
        ret = file.patch(
            name=str(files.math),
            source=patches.math_patch_template_http,
            template="jinja",
            context={"two": "two", "ten": 10},
            skip_verify=True,
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"


@pytest.mark.skip_on_freebsd(
    reason="Previously skipped on FreeBSD. Needs investigation as to why it currently False"
)
@pytest.mark.skip_on_darwin(
    reason="Failing and previously skipped because patch wasn't >= 2.6"
)
def test_patch_directory_remote_source_template(file, files, patches, subtests):
    """
    Test file.patch using a patch applied to a directory, with changes
    spanning multiple files, and the patch file coming from a remote
    source.
    """
    if salt.utils.platform.is_windows() and os.environ.get("PYTHONUTF8", "0") == "0":
        pytest.skip("Test will fail if PYTHONUTF8=1 is not set on windows")
    # Try without a source_hash and without skip_verify=True, this should
    # fail with a message about the source_hash
    with subtests.test("source_hash=None and skip_verify=False"):
        ret = file.patch(
            name=str(files.base_dir),
            source=patches.all_patch_template_http,
            template="jinja",
            context={"two": "two", "ten": 10},
            strip=1,
        )
        assert ret.result is False
        assert not ret.changes
        assert "Unable to verify upstream hash" in ret.comment

    # Re-run the state with a source hash, it should now succeed
    with subtests.test("source_hash!=None"):
        ret = file.patch(
            name=str(files.base_dir),
            source=patches.all_patch_template_http,
            source_hash=patches.all_patch_template_hash,
            template="jinja",
            context={"two": "two", "ten": 10},
            strip=1,
        )
        assert ret.result is True
        assert ret.changes
        assert ret.comment == "Patch successfully applied"
    with subtests.test("source_hash=None and skip_verify=True"):
        ret = file.patch(
            name=str(files.base_dir),
            source=patches.all_patch_template_http,
            template="jinja",
            context={"two": "two", "ten": 10},
            skip_verify=True,
            strip=1,
        )
        assert ret.result is True
        assert not ret.changes
        assert ret.comment == "Patch was already applied"


def test_patch_test_mode(file, files, patches):
    """
    Test file.patch using test=True
    """
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch, test=True)
    assert ret.result is None
    assert ret.changes
    assert ret.comment == "The patch would be applied"

    # Apply the patch for real. We'll then be able to test below that we
    # exit with a True rather than a None result if test=True is used on an
    # already-applied patch.
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch)
    assert ret.result is True
    assert ret.changes
    assert ret.comment == "Patch successfully applied"

    # Run again with test=True. Since the pre-check happens before we do
    # the __opts__['test'] check, we should exit with a True result just
    # the same as if we try to run this state on an already-patched file
    # *without* test=True.
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch, test=True)
    assert ret.result is True
    assert not ret.changes
    assert ret.comment == "Patch was already applied"

    # Empty the file to ensure that the patch doesn't apply cleanly
    files.numbers.write_text("")

    # Run again with test=True. Similar to the above run, we are testing
    # that we return before we reach the __opts__['test'] check. In this
    # case we should return a False result because we should already know
    # by this point that the patch will not apply cleanly.
    ret = file.patch(name=str(files.numbers), source=patches.numbers_patch, test=True)
    assert ret.result is False
    assert not ret.changes
    assert "Patch would not apply cleanly" in ret.comment



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>

    Test cases for salt.modules.haproxyconn
"""

import pytest

import salt.modules.haproxyconn as haproxyconn


class Mockcmds:
    """
    Mock of cmds
    """

    def __init__(self):
        self.backend = None
        self.server = None
        self.weight = None

    def listServers(self, backend):
        """
        Mock of listServers method
        """
        self.backend = backend
        return (
            "Name: server01 Status: UP Weight: 1 bIn: 22 bOut: 12\n"
            "Name: server02 Status: MAINT Weight: 2 bIn: 0 bOut: 0"
        )

    def enableServer(self, server, backend):
        """
        Mock of enableServer method
        """
        self.backend = backend
        self.server = server
        return "server enabled"

    def disableServer(self, server, backend):
        """
        Mock of disableServer method
        """
        self.backend = backend
        self.server = server
        return "server disabled"

    def getWeight(self, server, backend, weight=0):
        """
        Mock of getWeight method
        """
        self.backend = backend
        self.server = server
        self.weight = weight
        return "server weight"

    @staticmethod
    def showFrontends():
        """
        Mock of showFrontends method
        """
        return "frontend-alpha\nfrontend-beta\nfrontend-gamma"

    @staticmethod
    def showBackends():
        """
        Mock of showBackends method
        """
        return "backend-alpha\nbackend-beta\nbackend-gamma"


class Mockhaproxy:
    """
    Mock of haproxy
    """

    def __init__(self):
        self.cmds = Mockcmds()


class MockHaConn:
    """
    Mock of HaConn
    """

    def __init__(self, socket=None):
        self.ha_cmd = None

    def sendCmd(self, ha_cmd, objectify=False):
        """
        Mock of sendCmd method
        """
        self.ha_cmd = ha_cmd
        self.objectify = objectify
        return ha_cmd


@pytest.fixture
def configure_loader_modules():
    return {haproxyconn: {"haproxy": Mockhaproxy(), "_get_conn": MockHaConn}}


# 'list_servers' function tests: 1


def test_list_servers():
    """
    Test list_servers
    """
    assert haproxyconn.list_servers("mysql")


# 'enable_server' function tests: 1


def test_enable_server():
    """
    Test enable_server
    """
    assert haproxyconn.enable_server("web1.salt.com", "www")


# 'disable_server' function tests: 1


def test_disable_server():
    """
    Test disable_server
    """
    assert haproxyconn.disable_server("db1.salt.com", "mysql")


# 'get_weight' function tests: 1


def test_get_weight():
    """
    Test get the weight of a server
    """
    assert haproxyconn.get_weight("db1.salt.com", "mysql")


# 'set_weight' function tests: 1


def test_set_weight():
    """
    Test setting the weight of a given server
    """
    assert haproxyconn.set_weight("db1.salt.com", "mysql", weight=11)


# 'show_frontends' function tests: 1


def test_show_frontends():
    """
    Test print all frontends received from the HAProxy socket
    """
    assert haproxyconn.show_frontends()


def test_list_frontends():
    """
    Test listing all frontends
    """
    assert sorted(haproxyconn.list_frontends()) == sorted(
        ["frontend-alpha", "frontend-beta", "frontend-gamma"]
    )


# 'show_backends' function tests: 1


def test_show_backends():
    """
    Test print all backends received from the HAProxy socket
    """
    assert haproxyconn.show_backends()


def test_list_backends():
    """
    Test listing of all backends
    """
    assert sorted(haproxyconn.list_backends()) == sorted(
        ["backend-alpha", "backend-beta", "backend-gamma"]
    )


def test_get_backend():
    """
    Test get_backend and compare returned value
    """
    expected_data = {
        "server01": {"status": "UP", "weight": 1, "bin": 22, "bout": 12},
        "server02": {"status": "MAINT", "weight": 2, "bin": 0, "bout": 0},
    }
    assert haproxyconn.get_backend("test") == expected_data


def test_wait_state_true():
    """
    Test a successful wait for state
    """
    assert haproxyconn.wait_state("test", "server01")


def test_wait_state_false():
    """
    Test a failed wait for state, with a timeout of 0
    """
    assert not haproxyconn.wait_state("test", "server02", "up", 0)



"""
Tests the PIDfile deletion in the DaemonMixIn.
"""

import logging

import pytest

import salt.utils.parsers
from tests.support.mock import ANY, MagicMock, patch


@pytest.fixture
def daemon_mixin():
    mixin = salt.utils.parsers.DaemonMixIn()
    mixin.config = {"pidfile": "/some/fake.pid"}
    return mixin


def test_pid_file_deletion(daemon_mixin):
    """
    PIDfile deletion without exception.
    """
    with patch("os.unlink", MagicMock()) as unlink_mock:
        with patch("os.path.isfile", MagicMock(return_value=True)):
            with patch("salt.utils.parsers.log", MagicMock()) as log_mock:
                daemon_mixin._mixin_before_exit()
                unlink_mock.assert_called_once()
                log_mock.info.assert_not_called()
                log_mock.debug.assert_not_called()


def test_pid_deleted_oserror_as_root(daemon_mixin):
    """
    PIDfile deletion with exception, running as root.
    """
    with patch("os.unlink", MagicMock(side_effect=OSError())) as unlink_mock:
        with patch("os.path.isfile", MagicMock(return_value=True)):
            with patch("salt.utils.parsers.log", MagicMock()) as log_mock:
                if salt.utils.platform.is_windows():
                    patch_args = (
                        "salt.utils.win_functions.is_admin",
                        MagicMock(return_value=True),
                    )
                else:
                    patch_args = ("os.getuid", MagicMock(return_value=0))

                with patch(*patch_args):
                    daemon_mixin._mixin_before_exit()
                    assert unlink_mock.call_count == 1
                    log_mock.info.assert_called_with(
                        "PIDfile(%s) could not be deleted: %s",
                        format(daemon_mixin.config["pidfile"], ""),
                        ANY,
                        exc_info_on_loglevel=logging.DEBUG,
                    )


def test_pid_deleted_oserror_as_non_root(daemon_mixin):
    """
    PIDfile deletion with exception, running as non-root.
    """
    with patch("os.unlink", MagicMock(side_effect=OSError())) as unlink_mock:
        with patch("os.path.isfile", MagicMock(return_value=True)):
            with patch("salt.utils.parsers.log", MagicMock()) as log_mock:
                if salt.utils.platform.is_windows():
                    patch_args = (
                        "salt.utils.win_functions.is_admin",
                        MagicMock(return_value=False),
                    )
                else:
                    patch_args = ("os.getuid", MagicMock(return_value=1000))

                with patch(*patch_args):
                    daemon_mixin._mixin_before_exit()
                    assert unlink_mock.call_count == 1
                    log_mock.info.assert_not_called()
                    log_mock.debug.assert_not_called()



"""
:codeauthor: Herbert Buurman <herbert.buurman@ogd.nl>
"""

import datetime
import os
import textwrap

import pytest

# Import Salt Module
import salt.modules.acme as acme
import salt.utils.dictupdate
import salt.utils.platform
from salt.exceptions import SaltInvocationError
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {acme: {}}


def test_certs():
    """
    Test listing certs
    """
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "file.readdir": MagicMock(
                return_value=[".", "..", "README", "test_expired", "test_valid"]
            )
        },
    ), patch(
        "os.path.isdir",
        side_effect=lambda path: path
        in [
            os.path.join(acme.LE_LIVE, "test_expired"),
            os.path.join(acme.LE_LIVE, "test_valid"),
        ],
    ):
        assert acme.certs() == ["test_expired", "test_valid"]


def test_has():
    """
    Test checking if certificate (does not) exist.
    """
    with patch.dict(
        acme.__salt__, {"file.file_exists": MagicMock(return_value=True)}
    ):  # pylint: disable=no-member
        assert acme.has("test_expired")
    with patch.dict(
        acme.__salt__, {"file.file_exists": MagicMock(return_value=False)}
    ):  # pylint: disable=no-member
        assert not acme.has("test_invalid")


def test_needs_renewal():
    """
    Test if expired certs do indeed need renewal.
    """
    expired = (
        datetime.date.today() - datetime.timedelta(days=3) - datetime.date(1970, 1, 1)
    )
    valid = (
        datetime.date.today() + datetime.timedelta(days=3) - datetime.date(1970, 1, 1)
    )
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "tls.cert_info": MagicMock(
                return_value={"not_after": expired.total_seconds()}
            )
        },
    ):
        assert acme.needs_renewal("test_expired")
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "tls.cert_info": MagicMock(
                return_value={"not_after": valid.total_seconds()}
            )
        },
    ):
        assert not acme.needs_renewal("test_valid")
        # Test with integer window parameter
        assert acme.needs_renewal("test_valid", window=5)
        # Test with string-like window parameter
        assert acme.needs_renewal("test_valid", window="5")
        # Test with 'force' parameter
        assert acme.needs_renewal("test_valid", window="force")
        # Test with 'true' parameter
        assert acme.needs_renewal("test_valid", window=True)
        # Test with invalid window parameter
        pytest.raises(
            SaltInvocationError, acme.needs_renewal, "test_valid", window="foo"
        )


def test_expires():
    """
    Test if expires function functions properly.
    """
    test_value = datetime.datetime.today() - datetime.timedelta(days=3)
    test_stamp = test_value - datetime.datetime(1970, 1, 1)
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "tls.cert_info": MagicMock(
                return_value={"not_after": test_stamp.total_seconds()}
            )
        },
    ):
        assert (
            acme.expires("test_expired")
            == datetime.datetime.fromtimestamp(test_stamp.total_seconds()).isoformat()
        )


def test_info():
    """
    Test certificate information retrieval.
    """
    certinfo_tls_result = {
        "not_after": 1559471377,
        "signature_algorithm": "sha256WithRSAEncryption",
        "extensions": {},
        "fingerprint": (
            "FB:A4:5F:71:D6:5D:6C:B6:1D:2C:FD:91:09:2C:1C:52:"
            "3C:EC:B6:4D:1A:95:65:37:04:D0:E2:5E:C7:64:0C:9C"
        ),
        "serial_number": 6461481982668892235,
        "issuer": {},
        "not_before": 1559557777,
        "subject": {},
    }
    certinfo_x509_result = {
        "Not After": "2019-06-02 10:29:37",
        "Subject Hash": "54:3B:6C:A4",
        "Serial Number": "59:AB:CB:A0:FB:90:E8:4B",
        "SHA1 Finger Print": (
            "F1:8D:F3:26:1B:D3:88:32:CD:B6:FA:3B:85:58:DA:C7:6F:62:BE:7E"
        ),
        "SHA-256 Finger Print": (
            "FB:A4:5F:71:D6:5D:6C:B6:1D:2C:FD:91:09:2C:1C:52:"
            "3C:EC:B6:4D:1A:95:65:37:04:D0:E2:5E:C7:64:0C:9C"
        ),
        "MD5 Finger Print": "95:B5:96:9B:42:A5:9E:20:78:FD:99:09:4B:21:1E:97",
        "Version": 3,
        "Key Size": 2048,
        "Public Key": (
            "-----BEGIN PUBLIC KEY-----\n"
            "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAsVO2vwQPKU92PSBnuGid\n"
            "k8t6KWVE2jEBM10u7CgqQmD/JCnYflEHAo1nOsD7wxdhBrxhf5Qs+pEX1HOsh8VA\n"
            "HDTim0iE8nQVJ0Iuen2SrwaWMhwKmZTSJRYMgd46oCMi2RdlCvcgF2Hw6RTwF7FT\n"
            "hnksc4HBT91XddnP32N558tOT3YejafQNvClz5WcR+E0JzqGrV/+wfe3o+j/q5eK\n"
            "UowttWazeSMvuROtqj/fEk0rop4D14pgzZqWi30tjwhJNl6fSPFWBrLEHGNyDJ+O\n"
            "zfov0B2MRLJibH7GMkOCwsP2g1lVOReqcml+ju6zAKW8nHBTRg0iXB18Ifxef57Y\n"
            "AQIDAQAB\n"
            "-----END PUBLIC KEY-----\n"
        ),
        "Issuer": {},
        "Issuer Hash": "54:3B:6C:A4",
        "Not Before": "2019-06-03 10:29:37",
        "Subject": {},
    }

    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "file.file_exists": MagicMock(return_value=True),
            "tls.cert_info": MagicMock(return_value=certinfo_tls_result),
        },
    ):
        assert acme.info("test") == certinfo_tls_result
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "file.file_exists": MagicMock(return_value=True),
            "x509.read_certificate": MagicMock(return_value=certinfo_x509_result),
        },
    ):
        assert acme.info("test") == certinfo_x509_result
    with patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "file.file_exists": MagicMock(return_value=True),
            "cmd.run": MagicMock(return_value="foo"),
        },
    ):
        assert acme.info("test") == {"text": "foo"}


def test_cert():
    """
    Test certificate retrieval/renewal
    """
    valid_timestamp = (
        datetime.datetime.now()
        + datetime.timedelta(days=30)
        - datetime.datetime(1970, 1, 1, 0, 0, 0, 0)
    ).total_seconds()
    expired_timestamp = (
        datetime.datetime.now()
        - datetime.timedelta(days=3)
        - datetime.datetime(1970, 1, 1, 0, 0, 0, 0)
    ).total_seconds()
    cmd_new_cert = {
        "stdout": textwrap.dedent(
            """
            IMPORTANT NOTES:
             - Congratulations! Your certificate and chain have been saved at:
               /etc/letsencrypt/live/test/fullchain.pem
               Your key file has been saved at:
               /etc/letsencrypt/live/test/privkey.pem
               Your cert will expire on 2019-08-07. To obtain a new or tweaked
               version of this certificate in the future, simply run certbot
               again. To non-interactively renew *all* of your certificates, run
               "certbot renew"
             - If you like Certbot, please consider supporting our work by:

               Donating to ISRG / Let's Encrypt:   https://letsencrypt.org/donate
               Donating to EFF:                    https://eff.org/donate-le
            """
        ),
        "stderr": textwrap.dedent(
            """
            Saving debug log to /var/log/letsencrypt/letsencrypt.log
            Plugins selected: Authenticator standalone, Installer None
            Starting new HTTPS connection (1): acme-v02.api.letsencrypt.org
            Obtaining a new certificate
            Resetting dropped connection: acme-v02.api.letsencrypt.org
            """
        ),
        "retcode": 0,
    }
    result_new_cert = {
        "comment": "Certificate test obtained",
        "not_after": datetime.datetime.fromtimestamp(valid_timestamp).isoformat(),
        "changes": {"mode": "0640"},
        "result": True,
    }

    cmd_no_renew = {
        "stdout": textwrap.dedent(
            """
            - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            Certificate not yet due for renewal; no action taken.
            - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            """
        ),
        "stderr": textwrap.dedent(
            """Saving debug log to /var/log/letsencrypt/letsencrypt.log
            Plugins selected: Authenticator standalone, Installer None
            Starting new HTTPS connection (1): acme-v02.api.letsencrypt.org
            Cert not yet due for renewal
            Keeping the existing certificate
            """
        ),
        "retcode": 0,
    }
    if salt.utils.platform.is_freebsd():
        result_no_renew = {
            "comment": "Certificate "
            + os.path.join("/usr/local/etc/letsencrypt/live/test", "cert.pem")
            + " unchanged",
            "not_after": datetime.datetime.fromtimestamp(valid_timestamp).isoformat(),
            "changes": {},
            "result": True,
        }
    else:
        result_no_renew = {
            "comment": "Certificate "
            + os.path.join("/etc/letsencrypt/live/test", "cert.pem")
            + " unchanged",
            "not_after": datetime.datetime.fromtimestamp(valid_timestamp).isoformat(),
            "changes": {},
            "result": True,
        }
    result_renew = {
        "comment": "Certificate test renewed",
        "not_after": datetime.datetime.fromtimestamp(expired_timestamp).isoformat(),
        "changes": {},
        "result": True,
    }

    # Test fetching new certificate
    with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "cmd.run_all": MagicMock(return_value=cmd_new_cert),
            "file.file_exists": MagicMock(return_value=False),
            "tls.cert_info": MagicMock(return_value={"not_after": valid_timestamp}),
            "file.check_perms": MagicMock(
                side_effect=lambda a, x, b, c, d, follow_symlinks: (
                    salt.utils.dictupdate.set_dict_key_value(x, "changes:mode", "0640"),
                    None,
                )
            ),
        },
    ):
        assert acme.cert("test") == result_new_cert
        assert acme.cert("testing.example.com", certname="test") == result_new_cert
    # Test not renewing a valid certificate
    with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "cmd.run_all": MagicMock(return_value=cmd_no_renew),
            "file.file_exists": MagicMock(return_value=True),
            "tls.cert_info": MagicMock(return_value={"not_after": valid_timestamp}),
            "file.check_perms": MagicMock(
                side_effect=lambda a, x, b, c, d, follow_symlinks: (
                    salt.utils.dictupdate.set_dict_key_value(x, "result", True),
                    None,
                )
            ),
        },
    ):
        assert acme.cert("test") == result_no_renew
        assert acme.cert("testing.example.com", certname="test") == result_no_renew
    # Test renewing an expired certificate
    with patch("salt.modules.acme.LEA", "certbot"), patch.dict(
        acme.__salt__,
        {  # pylint: disable=no-member
            "cmd.run_all": MagicMock(return_value=cmd_new_cert),
            "file.file_exists": MagicMock(return_value=True),
            "tls.cert_info": MagicMock(return_value={"not_after": expired_timestamp}),
            "file.check_perms": MagicMock(
                side_effect=lambda a, x, b, c, d, follow_symlinks: (
                    salt.utils.dictupdate.set_dict_key_value(x, "result", True),
                    None,
                )
            ),
        },
    ):
        assert acme.cert("test") == result_renew
        assert acme.cert("testing.example.com", certname="test") == result_renew



import pytest

import salt.modules.pcs as pcs
from tests.support.mock import MagicMock, patch


class TstData:
    def __init__(self):
        self.nodea = "nodea"
        self.nodeb = "nodeb"
        self.nodes = [self.nodea, self.nodeb]
        self.extra_args = ["test", "extra", "args"]
        self.cluster_name = "testcluster"
        self.username = "hacluster"
        self.password = "hacluster"
        self.cib_filename = "/tmp/cib.xml"


@pytest.fixture
def test_data():
    return TstData()


@pytest.fixture
def configure_loader_modules():
    return {
        pcs: {"__salt__": {"pkg.version": MagicMock()}},
    }


@pytest.mark.parametrize("ver_cmp_ret,old_ver", [(1, False), (0, True)])
def test_auth(ver_cmp_ret, old_ver, test_data):
    """
    Test for authorising hosts on cluster
    """
    exp_cmd = ["pcs"]
    if old_ver:
        exp_cmd.extend(["cluster", "auth"])
    else:
        exp_cmd.extend(["host", "auth"])

    exp_cmd.extend(["-u", test_data.username, "-p", test_data.password])

    if old_ver:
        exp_cmd.extend(test_data.extra_args)

    exp_cmd.extend([test_data.nodea, test_data.nodeb])

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {
            "cmd.run_all": mock_cmd,
            "pkg.version_cmp": MagicMock(return_value=ver_cmp_ret),
        },
    )

    with patch_salt:
        pcs.auth(
            test_data.nodes,
            pcsuser=test_data.username,
            pcspasswd=test_data.password,
            extra_args=test_data.extra_args,
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_is_auth_old(test_data):
    """
    Test for checking it nodes are authorised.
    """
    exp_cmd = ["pcs", "cluster", "auth"]
    exp_cmd.extend(test_data.nodes)
    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {"cmd.run_all": mock_cmd, "pkg.version_cmp": MagicMock(return_value=0)},
    )

    with patch_salt:
        pcs.is_auth(
            test_data.nodes, pcsuser=test_data.username, pcspasswd=test_data.password
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_is_auth(test_data):
    """
    Test for checking it nodes are authorised.
    """
    exp_cmd = [
        "pcs",
        "host",
        "auth",
        "-u",
        test_data.username,
        "-p",
        test_data.password,
    ]
    exp_cmd.extend(test_data.nodes)

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {"cmd.run_all": mock_cmd, "pkg.version_cmp": MagicMock(return_value=1)},
    )

    with patch_salt:
        pcs.is_auth(
            test_data.nodes, pcsuser=test_data.username, pcspasswd=test_data.password
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


@pytest.mark.parametrize("ver_cmp_ret,old_ver", [(1, False), (0, True)])
def test_cluster_setup(ver_cmp_ret, old_ver, test_data):
    """
    Test for seting up a cluster
    """
    exp_cmd = ["pcs", "cluster", "setup"]
    if old_ver:
        exp_cmd.append("--name")
    exp_cmd = (
        exp_cmd + [test_data.cluster_name] + test_data.nodes + test_data.extra_args
    )

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {
            "cmd.run_all": mock_cmd,
            "pkg.version_cmp": MagicMock(return_value=ver_cmp_ret),
        },
    )

    with patch_salt:
        pcs.cluster_setup(
            test_data.nodes, test_data.cluster_name, extra_args=test_data.extra_args
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_cluster_destroy(test_data):
    """
    Test for destroying a cluster
    """
    exp_cmd = ["pcs", "cluster", "destroy"]
    exp_cmd.extend(test_data.extra_args)

    mock_cmd = MagicMock()
    patch_salt = patch.dict(pcs.__salt__, {"cmd.run_all": mock_cmd})
    with patch_salt:
        pcs.cluster_destroy(extra_args=test_data.extra_args)
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_cluster_node_add(test_data):
    """
    Test for adding a cluster
    """
    exp_cmd = ["pcs", "cluster", "node", "add"]
    exp_cmd = exp_cmd + [test_data.nodea] + test_data.extra_args

    mock_cmd = MagicMock()
    patch_salt = patch.dict(pcs.__salt__, {"cmd.run_all": mock_cmd})
    with patch_salt:
        pcs.cluster_node_add(test_data.nodea, extra_args=test_data.extra_args)
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_cib_push(test_data):
    """
    Test for pushing a CIB file
    """
    exp_cmd = [
        "pcs",
        "cluster",
        "cib-push",
        test_data.cib_filename,
        "scope=configuration",
    ]
    exp_cmd.extend(test_data.extra_args)

    mock_cmd = MagicMock()
    patch_salt = patch.dict(pcs.__salt__, {"cmd.run_all": mock_cmd})
    with patch_salt:
        pcs.cib_push(
            test_data.cib_filename,
            scope="configuration",
            extra_args=test_data.extra_args,
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


@pytest.mark.parametrize("ver_cmp_ret,old_ver", [(1, False), (0, True)])
def test_item_show_config_defaults(ver_cmp_ret, old_ver, test_data):
    """
    Test for item show
    """
    exp_cmd = ["pcs", "resource"]
    if old_ver:
        exp_cmd.append("show")
    else:
        exp_cmd.append("config")

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {
            "cmd.run_all": mock_cmd,
            "pkg.version_cmp": MagicMock(return_value=ver_cmp_ret),
        },
    )

    with patch_salt:
        pcs.item_show("resource")
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


@pytest.mark.parametrize("ver_cmp_ret,old_ver", [(1, False), (0, True)])
def test_item_show_set_itemid(ver_cmp_ret, old_ver, test_data):
    """
    Test for item show
    """
    exp_cmd = ["pcs", "resource"]
    if old_ver:
        exp_cmd.extend(["show", "itemid"])
    else:
        exp_cmd.extend(["config", "itemid"])

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {
            "cmd.run_all": mock_cmd,
            "pkg.version_cmp": MagicMock(return_value=ver_cmp_ret),
        },
    )

    with patch_salt:
        pcs.item_show("resource", "itemid")
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_item_show_set_itemid_config():
    """
    Test for item show
    """
    exp_cmd = ["pcs", "config", "show", "item_id"]
    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__, {"cmd.run_all": mock_cmd, "pkg.version_cmp": MagicMock(1)}
    )

    with patch_salt:
        pcs.item_show("config", item_id="item_id", item_type="item_type")
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_item_show_set_itemid_constraint():
    """
    Test for item show
    """
    exp_cmd = ["pcs", "constraint", "item_type", "show", "item_id", "--full"]

    mock_cmd = MagicMock()
    patch_salt = patch.dict(
        pcs.__salt__,
        {"cmd.run_all": mock_cmd, "pkg.version_cmp": MagicMock(1)},
    )

    with patch_salt:
        pcs.item_show("constraint", item_id="item_id", item_type="item_type")
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd


def test_item_create(test_data):
    """
    Test for item create
    """

    exp_cmd = [
        "pcs",
        "-f",
        test_data.cib_filename,
        "item",
        "create",
        "item_id",
        "item_type",
    ]
    exp_cmd.extend(test_data.extra_args)

    mock_cmd = MagicMock()
    patch_salt = patch.dict(pcs.__salt__, {"cmd.run_all": mock_cmd})
    with patch_salt:
        pcs.item_create(
            "item",
            "item_id",
            "item_type",
            create="create",
            extra_args=test_data.extra_args,
            cibfile=test_data.cib_filename,
        )
    assert mock_cmd.call_args_list[0][0][0] == exp_cmd
